import logging

from langchain.chat_models import init_chat_model
from langchain_core.messages import HumanMessage
from langchain_core.messages import SystemMessage

from .config import get_config

config = get_config()

log = logging.getLogger(__name__)


class Ai:
    def __init__(self):
        # google_genai google_vertexai
        provider = "google_genai" if config.env == "development" else "google_vertexai"
        
        self.model = init_chat_model(config.model, model_provider=provider)

    def invoke(self, messages):
        log.debug("message: %s", messages)
        response = self.model.invoke(messages)

        log.debug("response: %s", response)
        return response
    
    def structured_invoke(self, template, message):
        llm = self.model.with_structured_output(template)
        response = llm.invoke(message)

        log.debug("response: %s", response)
        return response
