import logging
import argparse
import configargparse

_config = None

def normalize_extensions(ext_list: list[str]) -> list[str]:
    # Normalizzo la lista estensioni
    default_ext=[".pdf", ".odt", ".txt", ".pptx", ".doc", ".xlsx", ".docx"]
    
    if not ext_list:
        return default_ext
    
    return [ext if ext.startswith('.') else f'.{ext}' for ext in ext_list]


def configure_logging(debug_flag: bool):
    logging.basicConfig(
        format="%(asctime)s [%(levelname)s] %(name)s %(message)s",
        level=logging.DEBUG if debug_flag else logging.INFO,
    )    

    if debug_flag:
        import jurigged
        logging.getLogger("watchdog").setLevel(logging.INFO)
        jurigged.watch(pattern="*gtagger*.py")



def get_config() -> argparse.Namespace:
    global _config

    if _config is not None:
        return _config

    parser = configargparse.get_argument_parser(
        config_file_parser_class=configargparse.YAMLConfigFileParser,
        formatter_class=configargparse.ArgumentDefaultsHelpFormatter,
    )
    parser.add_argument(
        "-c", "--config", required=False, is_config_file=True, help="config file path"
    )
    parser.add_argument(
        "--local", required=False, is_config_file=True, help="local config file path"
    )
    parser.add_argument(
        "--config-save",
        required=False,
        is_write_out_config_file_arg=True,
        help="config file path",
    )
    
    parser.add_argument("files", nargs="*", default=[], help="list of documents")
    parser.add_argument("--debug", action="store_true", default=False, help="debug")
    parser.add_argument("--dry-run", action="store_true", default=False, help="Check only")
    parser.add_argument("--task", default="report", choices=["upload", "report", "evaluate", "convert"], help="Desidered operation")
    parser.add_argument("--model", default="gemini-2.0-flash-lite-001", help="model")
    parser.add_argument("--person", action="append", default=["Federico Bricolo", "Maurizio Danese"], help="list of important people")
    parser.add_argument("--env", choices=["development", "production"], default="development", help="Running mode(development o production)")
    parser.add_argument("-e", "--extension", action="append", help="list of valuable extensions")
    parser.add_argument("-t", "--token", default="", help="paperless API token")

    _config, _ = parser.parse_known_args()
    
    configure_logging(_config.debug)
    _config.extension = normalize_extensions(_config.extension)

    return _config


