from datetime import datetime,timezone
import os
import random
import json
import time
import logging
from pathlib import Path

# richiede pip install httpx
import httpx
from http.cookies import SimpleCookie
from email.utils import parsedate_to_datetime

from .model import Correspondent, DocumentType
from .config import get_config

config = get_config()

#################from config import settings 
#from log_module import get_logger

logger = logging.getLogger(__name__)

# URL della pagina di login di Paperless da mettere in config.py
BASE_URL = "https://paper.csgalileo.org"
API_LOGIN = "/accounts/login/"
API_TAGS = "/api/tags/"
API_TASKID = "/api/tasks/"
API_DOCUMENT = "/api/documents/post_document/"
API_CORRESPONDENTS = "/api/correspondents/"
API_CUSTOM_FIELDS = "/api/custom_fields/"
API_DOCUMENT_TYPES = "/api/document_types/"
API_PATCH = "/api/documents/"

headers = {
        'Authorization': 'Token ' + config.token,
        'X-CSRFToken': 'ftP9XQusmf2eQqiPW0BaskHlmv1yfUmG',
        'Cookie': 'csrftoken=ftP9XQusmf2eQqiPW0BaskHlmv1yfUmG'
    }

####logger = get_logger(__name__, os.path.join(settings.get("working-path"),'paperless.log'),level=logging.DEBUG)

def generate_random_color():
    # Genera un colore a caso per i tag nuovi
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    
    return f"#{r:02x}{g:02x}{b:02x}"

# CUSTOM FIELDS MAPPING
# Evitiamo sorprese in caso di modifiche in Paperless
# chiave: nome attributo classe Document
# valore: campo "name" oggetto JSON
property_to_name_map = {
            "persone_citate": "persone citate",
            "argomenti": "argomenti",
            "anno": "anno",
            "tipo_evento": "tipo evento",
            "persone_evidenziate": "persone evidenziate",
            "relatori": "relatori",
            "persone_evidenziate": "persone evidenziate"
        }

# class Document:
#     def __init__(self, title):
#         self.title = title
#         self.persone_citate = []
#         self.argomenti = []
#         self.anno = 0
#         self.tipo_evento = ""
#         self.persone_evidenziate = []
#         self.relatori = []
#         self.tags = []


class Paperless:
    def __init__(self):
        self._token = tuple((None, None))
        

    def __str__(self):
        pass

    def csrf_is_expired(self):
        """
        Controlla se il token CSRF è scaduto.
        """
        token, expires = self._token

        if not token or not expires:  
            return True
        
        try:
            expires_datetime = parsedate_to_datetime(self._token[1])
            return expires_datetime <= datetime.now(timezone.utc)
        except Exception:
            logger.error(f"Formato di data non valido: {expires}")
            return True


    def get_csrf_token(self):
        with httpx.Client() as client:
            response = client.get(BASE_URL + API_LOGIN)

            if response.status_code != 200:
                logger.error(f"Errore: Status Code {response.status_code}")
                return 

            # Controllo se il token CSRF è nei cookie
            cookies = response.headers.get("set-cookie", "")
            token, expires = self.extract_csrf_token(cookies)
            
            if token:
                self._token = (token, expires)
                logger.info(f"CSRF Token trovato: {token}. Scadenza: {expires}")
            else:
                logger.warning("Nessun CSRF Token trovato!")
                

    def extract_csrf_token(self, set_cookie_header):
        """
        Estrae il token CSRF e la sua scadenza dall'header 'Set-Cookie'
        """
        cookie = SimpleCookie()
        cookie.load(set_cookie_header)

        if "csrftoken" in cookie:
            csrf_token = cookie["csrftoken"].value
            expires = cookie["csrftoken"]["expires"] if "expires" in cookie["csrftoken"] else None
            return csrf_token, expires
        return tuple(None, None)



    def post_document(self,filename, payload):
        # NOTA: il token adesso è cablato, altrimenti ogni run deve fare una request
        # 1 - recupera il CSRF token (è una chiamata a funzione, che lo rinnova nel caso non lo  fosse)
        # 2 - Costruisci l'header
        # 3 - fai la chiamata
        # 4 - se return code == 200 controlla l'esito con il task_id restiruito

        # Il tag deve esistere, altrimenti status code 400 (ok, per il momento non abbiamo tag)
        #                                  text =  {"tags":["Invalid pk \"10000\" - object does not exist."]}

        url = BASE_URL + API_DOCUMENT
        try:
            with open(filename, 'rb') as f:
                files = {'document': f}

                # data = {
                #     'title': title,
                #     'tags': tags,        
                # }
                
                
                with httpx.Client() as client:
                    response = client.post(url, headers=headers, files=files,data=payload)
                    if response.status_code == 200:
                        logger.info(f"Trying to upload document {filename} with name {payload["title"]}")
                        return response.json()
                    else:
                        logger.error(f"Failed to upload document. Status code: {response.status_code}, {response.text}")
                        return None
        except FileNotFoundError:
            logger.error(f"File {filename} non trovato.")
            return None
        except Exception as e:
            logger.error(f"Errore imprevisto in post_document: {e}")
            return None


    def taskid_check(self, task_id):
        # Abbiamo deciso di non fare una gestine asincrona, ma un POST alla volta.
        # Si rimane in questo codice fino che lo status è definitivo
        url = BASE_URL + API_TASKID
        params = {"task_id": f"{task_id}"}
        logger.debug(f"Check task: {task_id}")

        message = dict()

        while True:
            response = httpx.get(url, headers=headers, params=params)

            if response.status_code == 200:
                try: 
                    json_data = response.json()
                    # Ocio che se è None va in errore ( e mi è successo!!!)
                    if not json_data or not isinstance(json_data, list):
                            logger.error(f"Risposta inattesa dal server per task_id {task_id}: {json_data}")
                            return None
                    
                    result = response.json()[0].get("status")
                    if result is None:
                        return None
                    if result == "SUCCESS" or result == "FAILURE":
                        message = json_data[0]
                        logger.debug(f"Task {task_id} status: {result}")
                        break
                except (IndexError, KeyError, ValueError) as e:
                    logger.error(f"Errore nel parsing della risposta JSON per task_id {task_id}: {e}")
                    return None                
            else:
                logger.error(f"Errore {response.status_code}: {response.text}")
                message = {
                    "status": "FAILURE",
                    "status_code": response.status_code,
                }
                break
            time.sleep(3)
        return message

    @staticmethod
    def get_tags():
        response = httpx.get(BASE_URL + API_TAGS, headers=headers)

        if response.status_code == 200:
            tags = { tag["slug"]:tag["id"] for  tag in response.json()["results"] }
            return tags
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}

    @staticmethod
    def post_tag(tag_name, color=None):
        # Nota: il valore "matching_algorithm" = 0 disabilita gli automatismi
        #       nell'assegnazione dei tags. Altrimenti può succedere che alcuni tag
        #       vengano assegnati da paperless in modo arbitrario
        #       Tutti i tag che usiamo devono essere così, se vogliamo il pieno
        #       controllo.
        
        data = {
            "name": f"{tag_name}",
            "is_insensitive": True,
            "matching_algorithm": 0
        }

        if color:
            data["color"]= f"{color}"
        
        response = httpx.post(BASE_URL + API_TAGS, json=data, headers=headers)

        if response.status_code == 201:
            logger.debug("Risposta:", response.json())
            return response.json().get("id")
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")


    def load_tags(self):
        # TAG NORMALIZZATI
        # li preleviamo da un file json, risultato della classificazione IA
        #wp = os.path.join(settings.get("working-path"), settings.get("tag-file"))
        wp = os.path.join("/home/venom/Progetti/paperless-lab/importatore", "tags.json")
        classified_results = self.import_from_json(wp)

        # Estraggo i tags dall'oggetto
        # I tag non devono contenere spazi inizio e fine !!!!
        tags = set()
        for dizionario in classified_results:
            tags.update(dizionario["tags"])

        return tags


    def check_tags(self):
        # carica in un set la lista di tag normalizzata da IA
        document_tags = self.load_tags()
        logger.debug(f"tags normalizzati {document_tags}")
        
        # richiedi i tag di paperless in formato {chiave:valore}
        paperless_tags = self.get_tags()
        logger.debug(f"tags paperless {set(paperless_tags.keys())}")

        # differenza insiemistica: restano i tag che non esistono
        new_tags = document_tags - set(paperless_tags.keys())
        logger.debug(f"Trovati {len(new_tags)}"
                    f" nuovi tags {new_tags if len(new_tags) else ''}")

        # carica i nuovi tags in paperless
        if new_tags:
            logger.info(f"Aggiungo i nuovi tags: {new_tags} a paperless")
            for tag in new_tags:
                self.post_tag(tag, color=generate_random_color())

    @staticmethod
    def get_document_types():
        response = httpx.get(BASE_URL + API_DOCUMENT_TYPES, headers=headers)

        if response.status_code == 200:
            doc_types = [DocumentType(**item) for item in response.json()["results"]]
            return doc_types
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return []
    

    @staticmethod
    def get_correspondents():
        response = httpx.get(BASE_URL + API_CORRESPONDENTS, headers=headers)

        if response.status_code == 200:
            correspondents = [Correspondent(**item) for item in response.json()["results"]]
            return correspondents
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return []
    
    
    def get_custom_fields(self):
        response = httpx.get(BASE_URL + API_CUSTOM_FIELDS, headers=headers)

        if response.status_code == 200:
            logger.debug("get_custom_fields todo")
            return response.json()["results"]
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}

    def update_custom_fields(self, document_id, custom_fields):
        
        payload ={
            "custom_fields": [ {"field": i.id, "value":i.value} for i in custom_fields ]
        }
        
        logger.debug(f"update documento {document_id} con custom field {payload}")
        response = httpx.patch(BASE_URL + API_PATCH + str(document_id) + "/", json=payload, headers=headers)
        if response.status_code == 200:
            logger.debug("update_custom_fields todo")
            return response.json()
        else:
            logger.error(f"Errore {response.status_code}: {response.text}")
            return {}

    def import_from_json(self,filename):
        # Importa i dati contenuti in un file json e li ritrna come dizionario
        try:
            with open(filename) as f:
                return json.load(f)
        except json.JSONDecodeError:
            logger.error(f"File {filename} non contiene json valido.")
        except FileNotFoundError:
            logger.error(f"File {filename} non trovato.")
        except Exception as e:
            logger.error("Errore:", e)
        # Sa femo? Per il momento ritorno Non e non interrompo l'esecuzione
        return None
    

def main():
    paperless = Paperless()
    paperless.get_csrf_token()

    # # Dal file json di classificazione IA pechiamo i files e i tag da caricare
    # doc_list = import_from_json(os.path.join(settings.get("working-path"), settings.get("tag-file")))

    # # 3   - se c'e' un documento, upload
    # source_path = Path(os.path.join(settings.get("working-path"),
    #                          settings.get("consumption-directory") ))

    # # Ok, accrocchio, giusto per il test. Se il file non è presente scrivilo nel log e prosegui)

    # #for document in list(source_path.glob("*.pdf")):
    # for document in doc_list:
    #     filename = os.path.join(source_path, document["filename"])

    #     # devo costruire la lista dei tags
    #     tags = [ paperless_tags[tag] for tag in document["tags"]]

    #     task_id = post_document(filename, document["name"], tags)
    #     #task_id = asyncio.run(post_document1(filename, document["name"], tags))
    #     # Controllare None
    #     result = taskid_check(task_id)
    #     # SUCCESS siamo tutti contenti
    #     if result["status"] == "SUCCESS":
    #         # Sposta il documento in "success-directory"
    #         dest_path = Path(os.path.join(settings.get("working-path"),
    #                          settings.get("success-directory"),
    #                          document["filename"] ))
            
    #         #os.rename(filename, dest_path)
    #     else:
    #         # Se c'e' lo status_code significa che è fallita la chiamata
    #         if result.get("status_code"):
    #             pass
    #         else:
    #             # Sposta il documento in "failure-directory"
    #             dest_path = Path(os.path.join(settings.get("working-path"),
    #                             settings.get("failure-directory"),
    #                             document["filename"] ))
            
    #             #os.rename(filename, dest_path)               


if __name__ == "__main__":
    
    main()
    



